@@ -2004,7 +2004,7 @@ class ConfigObj(Section):
             # might need to encode
             # NOTE: This will *screw* UTF16, each line will start with the BOM
             if self.encoding:
-                out = [l.encode(self.encoding) for l in out]
+                out = [line.encode(self.encoding) for line in out]
             if (self.BOM and ((self.encoding is None) or
                               (BOM_LIST.get(self.encoding.lower()) == 'utf_8'))):
                 # Add the UTF8 BOM
@@ -2082,7 +2082,7 @@ class ConfigObj(Section):
             if preserve_errors:
                 # We do this once to remove a top level dependency on the validate module
                 # Which makes importing configobj faster
-                from configobj.validate import VdtMissingValue
+                from .validate import VdtMissingValue
                 self._vdtMissingValue = VdtMissingValue
 
             section = self
