@@ -1,6 +1,8 @@
 # Copyright (c) 2017 The Regents of the University of Michigan
 # All rights reserved.
 # This software is licensed under the BSD 3-Clause License.
+"""Contrib submodule containing Project class and indexing features."""
+
 import logging
 
 from . import indexing
@@ -11,6 +13,7 @@ from .indexing import BaseCrawler
 from .indexing import RegexFileCrawler
 from .indexing import JSONCrawler
 from .indexing import SignacProjectCrawler
+from .indexing import MainCrawler
 from .indexing import MasterCrawler
 from .indexing import fetch
 from .indexing import fetched
@@ -29,7 +32,7 @@ __all__ = [
     'indexing',
     'Project', 'TemporaryProject', 'get_project', 'init_project', 'get_job',
     'BaseCrawler', 'RegexFileCrawler', 'JSONCrawler', 'SignacProjectCrawler',
-    'MasterCrawler', 'fetch', 'fetched',
+    'MainCrawler', 'MasterCrawler', 'fetch', 'fetched',
     'export_one', 'export', 'export_to_mirror', 'export_pymongo',
     'index_files', 'index',
     'Collection',
