@@ -159,7 +159,7 @@ class RegexFileCrawler(BaseCrawler):
         MyCrawler.define('.*\/a_(?P<a>\d+)\.txt', 'TextFile')
     """
     "Mapping of compiled regex objects and associated formats."
-    definitions = dict()
+    definitions = dict()  # type: ignore
 
     @classmethod
     @deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
@@ -431,8 +431,8 @@ class SignacProjectCrawler(RegexFileCrawler):
 
 
 # this class is deprecated
-class MasterCrawler(BaseCrawler):
-    r"""Compiles a master index from indexes defined in access modules.
+class MainCrawler(BaseCrawler):
+    r"""Compiles a main index from indexes defined in access modules.
 
     An instance of this crawler will search the data space for access
     modules, which by default are named ``signac_access.py``. Once such
@@ -456,7 +456,7 @@ class MasterCrawler(BaseCrawler):
         def get_crawlers(root):
             yield MyCrawler(root)
 
-    In case that the master crawler has tags, the ``get_indexes()`` function
+    In case that the main crawler has tags, the ``get_indexes()`` function
     will always be ignored while crawlers yielded from the ``get_crawlers()``
     function will only be executed in case that they match at least one
     of the tags.
@@ -496,7 +496,7 @@ class MasterCrawler(BaseCrawler):
                 details="The indexing module is deprecated.")
     def __init__(self, root, raise_on_error=False):
         self.raise_on_error = raise_on_error
-        super(MasterCrawler, self).__init__(root=root)
+        super(MainCrawler, self).__init__(root=root)
 
     def _docs_from_module(self, dirpath, fn):
         name = os.path.join(dirpath, fn)
@@ -536,7 +536,7 @@ class MasterCrawler(BaseCrawler):
 
         if hasattr(module, 'get_crawlers'):
             for crawler in module.get_crawlers(dirpath):
-                logger.info("Executing slave crawler:\n {}".format(crawler))
+                logger.info("Executing subcrawler:\n {}".format(crawler))
                 if _check_tags(getattr(crawler, 'tags', None)):
                     for doc in crawler.crawl():
                         doc.setdefault(
@@ -544,7 +544,7 @@ class MasterCrawler(BaseCrawler):
                         yield doc
 
     def docs_from_file(self, dirpath, fn):
-        """Compile master index from file in case it is an access module.
+        """Compile main index from file in case it is an access module.
 
         :param dirpath: The path of the file relative to root.
         :param fn: The filename of the file.
@@ -563,6 +563,16 @@ class MasterCrawler(BaseCrawler):
                 logger.debug("Completed indexing from '{}'.".format(os.path.join(dirpath, fn)))
 
 
+# Deprecated API
+class MasterCrawler(MainCrawler):
+    def __init__(self, *args, **kwargs):
+        warnings.warn(
+            "The MasterCrawler class has been replaced by the MainCrawler class. "
+            "Both classes are deprecated and will be removed in a future release.",
+            DeprecationWarning)
+        super(MasterCrawler, self).__init__(*args, **kwargs)
+
+
 @deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
             details="The indexing module is deprecated.")
 def fetch(doc_or_id, mode='r', mirrors=None, num_tries=3, timeout=60, ignore_local=False):
@@ -931,9 +941,9 @@ def index_files(root='.', formats=None, depth=0):
 @deprecated(deprecated_in="1.3", removed_in="2.0", current_version=__version__,
             details="The indexing module is deprecated.")
 def index(root='.', tags=None, depth=0, **kwargs):
-    r"""Generate a master index.
+    r"""Generate a main index.
 
-    A master index is compiled from other indexes by searching
+    A main index is compiled from other indexes by searching
     for modules named ``signac_access.py`` and compiling all
     indexes which are yielded from a function ``get_indexes(root)``
     defined within that module as well as the indexes generated by
@@ -950,21 +960,21 @@ def index(root='.', tags=None, depth=0, **kwargs):
             yield signac.index_files(root, r'.*\.txt')
 
     Internally, this function constructs an instance of
-    :py:class:`.MasterCrawler` and all extra key-word arguments
-    will be forwarded to the constructor of said master crawler.
+    :py:class:`.MainCrawler` and all extra key-word arguments
+    will be forwarded to the constructor of said main crawler.
 
     :param root: Look for access modules under this directory path.
     :type root: str
-    :param tags: If tags are provided, do not execute slave crawlers
+    :param tags: If tags are provided, do not execute subcrawlers
         that don't match the same tags.
     :param depth: Limit the search to the specified directory depth.
     :param kwargs: These keyword-arguments are forwarded to the
-        internal MasterCrawler instance.
+        internal MainCrawler instance.
     :type depth: int
-    :yields: The master index documents as instances of dict.
+    :yields: The main index documents as instances of dict.
     """
 
-    class Crawler(MasterCrawler):
+    class Crawler(MainCrawler):
         pass
 
     if tags is not None:
