@@ -1,9 +1,11 @@
 # Copyright (c) 2018 The Regents of the University of Michigan
 # All rights reserved.
 # This software is licensed under the BSD 3-Clause License.
+"""Wrapper around json parsing library."""
 import logging
 from json import load, loads, JSONEncoder
 from json.decoder import JSONDecodeError
+from typing import Any, Dict, Optional
 
 logger = logging.getLogger(__name__)
 
@@ -21,7 +23,7 @@ class CustomJSONEncoder(JSONEncoder):
     an object that is otherwise not serializable, by calling the object's
     `_as_dict()` method.
     """
-    def default(self, o):
+    def default(self, o: Any) -> Dict[str, Any]:
         if NUMPY:
             if isinstance(o, numpy.number):
                 return o.item()
@@ -35,7 +37,8 @@ class CustomJSONEncoder(JSONEncoder):
             return super(CustomJSONEncoder, self).default(o)
 
 
-def dumps(o, sort_keys=False, indent=None):
+def dumps(o: Any, sort_keys: bool = False, indent: Optional[int] = None) -> str:
+    """Convert a JSON-compatible mapping into a string."""
     return CustomJSONEncoder(sort_keys=sort_keys, indent=indent).encode(o)
 
 
